﻿using System.ComponentModel.DataAnnotations;

namespace IMSES_Template.Model
{
    public class Hierarchy : BaseGroup
    {
        private Course _attachedCourse;
        
        /// <summary>
        /// The attached course, which for normal hierarchies will be null.
        /// </summary>
        public Course AttachedCourse
        {
            get { return _attachedCourse; }
            set 
            { 
                // Check if the SyncIds match
                if (!value.SyncId.Equals(SyncId))
                {
                    throw new ValidationException("The Linked Course does not have the same SyncId as the Hierarchy!");
                }
                _attachedCourse = value;
                GroupType = GroupType.Course;
            }
        }

        /// <summary>
        /// When creating a Hierarchy object the SyncId is required.
        /// </summary>
        /// <param name="syncId">The SyncId of the Hierarchy.</param>
        public Hierarchy(string syncId)
            : this(syncId, false)
        {
        }

        /// <summary>
        /// When creating a Hierarchy object the SyncId is required.
        /// </summary>
        /// <param name="syncId">The SyncId of the Hierarchy.</param>
        /// <param name="isSchool">True/False if the Hierarchy is a School.</param>
        public Hierarchy(string syncId, bool isSchool)
        {
            SyncId = syncId;
            GroupType = isSchool ? GroupType.School : GroupType.Hierarchy;
            VisibleInSearch = true;
        }

        /// <summary>
        /// Checks if the object is valid, i.e. that all properties marked with [Required] are actually initialized and filled, 
        /// and throws an exception with the porperties that aren't. It also checks the Attached Course, if one is set.
        /// </summary>
        public new void CheckRequired()
        {
            base.CheckRequired();

            if (AttachedCourse != null)
                AttachedCourse.CheckSingleRequired();
        }

        /// <summary>
        /// Checks if the object is valid, i.e. that all properties marked with [Required] are actually initialized and filled, 
        /// and throws an exception with the porperties that aren't. It also checks the Attached Course, if one is set.
        /// </summary>
        public void CheckSingleRequired()
        {
            base.CheckRequired();
        }


        /// <summary>
        /// Checks if all required fields are filled in, and checks if certain properties are not too long. It also checks the Attached Course, if set.
        /// </summary>
        /// <returns>Either the properties that are required and not filled in, or an error about the length of an property.</returns>
        public new void CheckValid()
        {
            CheckSingleValid();

            if (AttachedCourse != null)
                AttachedCourse.CheckSingleValid();
        }

    }

    
}
