﻿using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;

namespace IMSES_Template.Model
{
    public class User : BaseEntity
    {
        /// <summary>
        ///  The SyncKey of the User, also known as Synchronisation Key.
        /// </summary>
        [Required]
        public string SyncKey { get; protected set; }

        /// <summary>
        ///  First Name of the User.
        /// </summary>
        [Required]
        public string FirstName { get; set; }

        /// <summary>
        /// Last Name of the User.
        /// </summary>
        [Required]
        public string LastName { get; set; }

        /// <summary>
        /// Username of the User (inlog name).
        /// </summary>
        [Required]
        public string UserName { get; set; }

        /// <summary>
        /// Profile of the User (see below for options.)
        /// </summary>
        [Required] 
        public Profile Profile { get; set; }

        /// <summary>
        /// The email address of the User.
        /// </summary>
        public string Email { get; set; }

        /// <summary>
        /// The address of the User, split into four parts (see below).
        /// </summary>
        public Address Address { get; set; }

        /// <summary>
        /// The Birthdate of the User.
        /// </summary>
        public System.DateTime BirthDate { get; set; }

        /// <summary>
        /// Cell Phone number of the User (also known as Cellular phone).
        /// </summary>
        public string CellPhone { get; set; }

        /// <summary>
        /// Phone number for the User.
        /// </summary>
        public string Phone { get; set; }

        /// <summary>
        /// Nickname of the User (shown in chats)
        /// </summary>
        public string NickName { get; set; }

        /// <summary>
        /// Password of the User.
        /// </summary>
        public string Password { get; set; }


        /// <summary>
        /// The list with children of this User, only use when the Parent portal is enabled!
        /// </summary>
        public List<string> Children { get; set; }

        /// <summary>
        /// Collection of Custom Strings.
        /// </summary>
        public CustomStrings CustomStrings { get; set; }

        /// <summary>
        /// Setup information about the Email Server the User uses.
        /// </summary>
        public EmailServer EmailServer { get; set; }

        /// <summary>
        /// The MetaData of the User.
        /// </summary>
        public List<MetaData> MetaData { get; set; } 

        /// <summary>
        /// When creating an User object the SyncKey is required.
        /// </summary>
        /// <param name="syncKey">The SyncKey of the User.</param>
        public User(string syncKey)
        {
            SyncKey = syncKey;
        }


        /// <summary>
        /// Checks if the user is valid, i.e. that all properties marked with [Required] are actually initialized and filled, 
        /// and throws an exception with the porperties that aren't.
        /// </summary>
        public void CheckRequired()
        {
            CheckRequired(SyncKey);
        }

        /// <summary>
        /// Checks if certain properties are not too long.
        /// </summary>
        public void CheckValid()
        {
            // Check if certain properties are not too long.
            if (SyncKey.Length > 255)
                throw new ValidationException(string.Format("({0}) SyncKey is too long, max length is 255 characters.", SyncKey));
            
            if (FirstName != null && FirstName.Length > 64)
                throw new ValidationException(string.Format("({0}) FirstName is too long, max length is 64 characters.", SyncKey));
            
            if (LastName != null && LastName.Length > 64)
                throw new ValidationException(string.Format("({0}) LastName is too long, max length is 64 characters.", SyncKey));

            if (UserName != null && UserName.Length > 64)
                throw new ValidationException(string.Format("({0}) UserName is too long, max length is 64 characters.", SyncKey));

            if (NickName != null && NickName.Length > 32)
                throw new ValidationException(string.Format("({0}) NickName is too long, max length is 32 characters.", SyncKey));
        }


    }

    public class Address
    {
        public string AddressLine1 { get; set; }
        public string AddressLine2 { get; set; }
        public string PostalCode { get; set; }
        public string City { get; set; }
    }

    public class CustomStrings
    {
        public string CustomString1 { get; set; }
        public string CustomString2 { get; set; }
        public string CustomString3 { get; set; }
        public string CustomString4 { get; set; }
        public string CustomString5 { get; set; }
    }

    public class EmailServer
    {
        public string Email { get; set; }
        public string Server { get; set; }
        public string Port { get; set; }
        public string Type { get; set; }
        public string UserId { get; set; }
        public string Password { get; set; }
    }

    public class MetaData
    {
        public string Type { get; set; }
        public string Value { get; set; }
    }
}